function filter(target, callback, options, thisArg) {
  'use strict';

  if (!target) {
    return target;
  }

  if (typeof callback !== 'function') {
    throw new TypeError('\'callback\' has to be a function');
  }

  if (Array.isArray(target)) {
    return target.filter(callback, thisArg);
  }

  options = {...options};
  const hasOwn = Object.prototype.hasOwnProperty;
  const root = target;

  function _filter(target, path) {
    // Const obj = Object(target);
    const obj = target;
    const res = {};

    for (const key in obj) {
      if (!hasOwn.call(obj, key)) {
        continue;
      }

      let kValue = obj[key];
      if (options.recursive && kValue && typeof kValue === 'object') {
        if (Array.isArray(kValue)) {
          if (options.filterArray) {
            kValue = kValue.filter(callback, thisArg);
          }
        } else {
          kValue = _filter(kValue, path + '/' + key);
        }
      }

      if (callback.call(thisArg, kValue, key, obj, root, path + '/' + key)) {
        res[key] = kValue;
      }
    }

    return res;
  }

  return _filter(target, '');
}

module.exports = filter;
